package com.ruoyi.mall.service;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

import com.ruoyi.mall.domain.MallOrderBack;
import com.ruoyi.mall.domain.MallOrderOrder;

/**
 * 退款单Service接口
 *
 * @author chang
 * @date 2021-11-25
 */
public interface IMallOrderBackService
{
    /**
     * 查询退款单
     *
     * @param id 退款单主键
     * @return 退款单
     */
    public MallOrderBack selectMallOrderBackById(Long id);

    /**
     * 查询退款单列表
     *
     * @param mallOrderBack 退款单
     * @return 退款单集合
     */
    public List<MallOrderBack> selectMallOrderBackList(MallOrderBack mallOrderBack);

    /**
     * 新增退款单
     *
     * @param mallOrderBack 退款单
     * @return 结果
     */
    public int insertMallOrderBack(MallOrderBack mallOrderBack);

    /**
     * 修改退款单
     *
     * @param mallOrderBack 退款单
     * @return 结果
     */
    public int updateMallOrderBack(MallOrderBack mallOrderBack);

    /**
     * 批量删除退款单
     *
     * @param ids 需要删除的退款单主键集合
     * @return 结果
     */
    public int deleteMallOrderBackByIds(Long[] ids);

    /**
     * 删除退款单信息
     *
     * @param id 退款单主键
     * @return 结果
     */
    public int deleteMallOrderBackById(Long id);


    // /*
    //  * 申请退款
    //  * */
    // public int applyForRefund(Map<String, Object> params);

    /*
     * 申请退款
     * */
    public int applyForRefund(MallOrderOrder mallOrderOrder);

    /*
     * 申请退款
     * */
    public int cancleRefund(Map<String, Object> params);

    /*
    * 同意、拒绝退款
    * */
    public int handleRefund(Map<String, Object> params);

    public List<MallOrderBack> selectMallOrderBackByOrderId(Long orderId);

    /**
     * 退货金额
     *
     * @param orderId 订单ID
     * @param status 状态
     * @return 结果
     */
    public BigDecimal getReturnPrice(Long orderId, String status);
}
